/*!	\file ks0073.c
	\brief AVR-GCC library for interfacing a KS0073 based LCD via hardware SPI
	
	Author: J. Gerlach - gerlach_joerg(at)web.de
*/
#include "ks0073.h"

// static Prototypes

/*! Send Byte via Hardware-SPI to KS0073 controller
	sends data byte to LCD, data will be split up in nibbles	
*/
static void lcd_put_byte(uint8_t data);

/*! former macro to send byte via SPI
	is being used to send startbyte
*/
static  uint8_t spi_put_byte(uint8_t c);

/*!
	\fn static uint8_t lcd_get_data(uint8_t adress);
	\brief not implemented! - read byte from LCD
	\param adress 7 Byte DDRAM address
*/

// lcd_ functions (description in ks0073.h)

static uint8_t spi_put_byte(uint8_t c)
{
	SPDR = c;
	while(!(SPSR & (1<<SPIF)));
	return SPDR;
}

static void lcd_put_byte(uint8_t data)
{
	uint8_t high_nibble = (data >> 4);
	data &= 0x0F;
	spi_put_byte(data);
	spi_put_byte(high_nibble);
}

void lcd_put_cmd(uint8_t cmd)
{	
	spi_put_byte(STARTBYTE);
	lcd_put_byte(cmd);
	return;
}

void lcd_put_data(uint8_t data)
{
	spi_put_byte(STARTBYTE|(1<<LCD_RS));
	lcd_put_byte(data);
	return;
}

void lcd_puts(char *s)
{
	CLR_CS();
	do{
		lcd_put_data(*s);
		s++;
		LCD_BUSYWAIT();
	}
	while(*s);
	SET_CS();
	return;
}

void lcd_puts_p(const char *progmem_s )
{
    register char c;
    CLR_CS();
	// GCC simply doesn't like AVR's Havard-architecture
    while((c = pgm_read_byte(progmem_s++))){
		lcd_put_data(c);
		LCD_BUSYWAIT();
	}
	SET_CS();
	return;
}

void lcd_gotoxy(uint8_t col, uint8_t row)
{
	uint8_t pos = 0x80 | (row*(0x80/DISPLAYROWS) + col);
	CLR_CS();
	lcd_put_cmd(pos);
	LCD_BUSYWAIT();
	SET_CS();
	return;
}

/*
	read data from display
*/ /*
static uint8_t lcd_get_data(uint8_t adress)
{	
	// TODO: complete if needed
	return ;
}
*/

uint8_t lcd_get_status(void)
{
	spi_put_byte(STARTBYTE|(1<<LCD_RW));
	return spi_put_byte(DUMMYBYTE);	//Dummy write, anything without five 1s in a row
	
}

void lcd_init(void)
{	
	uint8_t i;
	STARTUPDELAY();
	CLR_CS();
	lcd_put_cmd(FUNCSET|LCD_DL);	// 8Bit, no RE
	LCD_BUSYWAIT();
	lcd_put_cmd(ENTRYMODE|LCD_INC);	// auto inc
	LCD_BUSYWAIT();
	lcd_put_cmd(FUNCSET|LCD_DL|LCD_RE|LCD_DH);	// 8Bit, set RE, DH is not documented in Samsung datasheet
	LCD_BUSYWAIT();
	lcd_put_cmd(ENH_EXT_FUNCT|LCD_NW);	// 4 row mode
	LCD_BUSYWAIT();
	lcd_put_cmd(ENH_SET_SEGA); // set SEGRAM ("icon-RAM") adresse to 0
	LCD_BUSYWAIT();
	for(i=16; i>0; i--){ // clear all 16 icons on EA204-6 Display, may not be needed on others
		lcd_put_data(0);
		LCD_BUSYWAIT();
	}
	lcd_put_cmd(ENH_FUNCSET|LCD_DL); // 8Bit, clear RE
	LCD_BUSYWAIT();
	lcd_put_cmd(DISP_ON|LCD_DISP|LCD_CUR|LCD_BLINK); // turn on display, cursor, cursor blink
	LCD_BUSYWAIT();
	lcd_put_cmd(CLEARDISP);
	LCD_BUSYWAIT();
	SET_CS();
	return;
}
